package emulation

// Code generated by cdproto-gen. DO NOT EDIT.

// EventVirtualTimeBudgetExpired notification sent after the virtual time
// budget for the current VirtualTimePolicy has run out.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/Emulation#event-virtualTimeBudgetExpired
type EventVirtualTimeBudgetExpired struct{}
