package input

// Code generated by cdproto-gen. DO NOT EDIT.

// EventDragIntercepted emitted only when Input.setInterceptDrags is enabled.
// Use this data with Input.dispatchDragEvent to restore normal drag and drop
// behavior.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/Input#event-dragIntercepted
type EventDragIntercepted struct {
	Data *DragData `json:"data"`
}
