package fedcm

// Code generated by cdproto-gen. DO NOT EDIT.

// EventDialogShown [no description].
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/FedCm#event-dialogShown
type EventDialogShown struct {
	DialogID   string     `json:"dialogId"`
	DialogType DialogType `json:"dialogType"`
	Accounts   []*Account `json:"accounts"`
	Title      string     `json:"title"` // These exist primarily so that the caller can verify the RP context was used appropriately.
	Subtitle   string     `json:"subtitle,omitempty"`
}

// EventDialogClosed triggered when a dialog is closed, either by user
// action, JS abort, or a command below.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/FedCm#event-dialogClosed
type EventDialogClosed struct {
	DialogID string `json:"dialogId"`
}
