package log

// Code generated by cdproto-gen. DO NOT EDIT.

// EventEntryAdded issued when new message was logged.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/Log#event-entryAdded
type EventEntryAdded struct {
	Entry *Entry `json:"entry"` // The entry.
}
