package serviceworker

// Code generated by cdproto-gen. DO NOT EDIT.

// EventWorkerErrorReported [no description].
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/ServiceWorker#event-workerErrorReported
type EventWorkerErrorReported struct {
	ErrorMessage *ErrorMessage `json:"errorMessage"`
}

// EventWorkerRegistrationUpdated [no description].
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/ServiceWorker#event-workerRegistrationUpdated
type EventWorkerRegistrationUpdated struct {
	Registrations []*Registration `json:"registrations"`
}

// EventWorkerVersionUpdated [no description].
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/ServiceWorker#event-workerVersionUpdated
type EventWorkerVersionUpdated struct {
	Versions []*Version `json:"versions"`
}
