package audits

// Code generated by cdproto-gen. DO NOT EDIT.

// EventIssueAdded [no description].
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/Audits#event-issueAdded
type EventIssueAdded struct {
	Issue *InspectorIssue `json:"issue"`
}
