package database

// Code generated by cdproto-gen. DO NOT EDIT.

// EventAddDatabase [no description].
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/Database#event-addDatabase
type EventAddDatabase struct {
	Database *Database `json:"database"`
}
