// Code generated by easyjson for marshaling/unmarshaling. DO NOT EDIT.

package webauthn

import (
	json "encoding/json"
	easyjson "github.com/mailru/easyjson"
	jlexer "github.com/mailru/easyjson/jlexer"
	jwriter "github.com/mailru/easyjson/jwriter"
)

// suppress unused package warning
var (
	_ *json.RawMessage
	_ *jlexer.Lexer
	_ *jwriter.Writer
	_ easyjson.Marshaler
)

func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn(in *jlexer.Lexer, out *VirtualAuthenticatorOptions) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "protocol":
			(out.Protocol).UnmarshalEasyJSON(in)
		case "ctap2Version":
			(out.Ctap2version).UnmarshalEasyJSON(in)
		case "transport":
			(out.Transport).UnmarshalEasyJSON(in)
		case "hasResidentKey":
			out.HasResidentKey = bool(in.Bool())
		case "hasUserVerification":
			out.HasUserVerification = bool(in.Bool())
		case "hasLargeBlob":
			out.HasLargeBlob = bool(in.Bool())
		case "hasCredBlob":
			out.HasCredBlob = bool(in.Bool())
		case "hasMinPinLength":
			out.HasMinPinLength = bool(in.Bool())
		case "hasPrf":
			out.HasPrf = bool(in.Bool())
		case "automaticPresenceSimulation":
			out.AutomaticPresenceSimulation = bool(in.Bool())
		case "isUserVerified":
			out.IsUserVerified = bool(in.Bool())
		case "defaultBackupEligibility":
			out.DefaultBackupEligibility = bool(in.Bool())
		case "defaultBackupState":
			out.DefaultBackupState = bool(in.Bool())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn(out *jwriter.Writer, in VirtualAuthenticatorOptions) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"protocol\":"
		out.RawString(prefix[1:])
		(in.Protocol).MarshalEasyJSON(out)
	}
	if in.Ctap2version != "" {
		const prefix string = ",\"ctap2Version\":"
		out.RawString(prefix)
		(in.Ctap2version).MarshalEasyJSON(out)
	}
	{
		const prefix string = ",\"transport\":"
		out.RawString(prefix)
		(in.Transport).MarshalEasyJSON(out)
	}
	if in.HasResidentKey {
		const prefix string = ",\"hasResidentKey\":"
		out.RawString(prefix)
		out.Bool(bool(in.HasResidentKey))
	}
	if in.HasUserVerification {
		const prefix string = ",\"hasUserVerification\":"
		out.RawString(prefix)
		out.Bool(bool(in.HasUserVerification))
	}
	if in.HasLargeBlob {
		const prefix string = ",\"hasLargeBlob\":"
		out.RawString(prefix)
		out.Bool(bool(in.HasLargeBlob))
	}
	if in.HasCredBlob {
		const prefix string = ",\"hasCredBlob\":"
		out.RawString(prefix)
		out.Bool(bool(in.HasCredBlob))
	}
	if in.HasMinPinLength {
		const prefix string = ",\"hasMinPinLength\":"
		out.RawString(prefix)
		out.Bool(bool(in.HasMinPinLength))
	}
	if in.HasPrf {
		const prefix string = ",\"hasPrf\":"
		out.RawString(prefix)
		out.Bool(bool(in.HasPrf))
	}
	if in.AutomaticPresenceSimulation {
		const prefix string = ",\"automaticPresenceSimulation\":"
		out.RawString(prefix)
		out.Bool(bool(in.AutomaticPresenceSimulation))
	}
	if in.IsUserVerified {
		const prefix string = ",\"isUserVerified\":"
		out.RawString(prefix)
		out.Bool(bool(in.IsUserVerified))
	}
	if in.DefaultBackupEligibility {
		const prefix string = ",\"defaultBackupEligibility\":"
		out.RawString(prefix)
		out.Bool(bool(in.DefaultBackupEligibility))
	}
	if in.DefaultBackupState {
		const prefix string = ",\"defaultBackupState\":"
		out.RawString(prefix)
		out.Bool(bool(in.DefaultBackupState))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v VirtualAuthenticatorOptions) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v VirtualAuthenticatorOptions) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *VirtualAuthenticatorOptions) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *VirtualAuthenticatorOptions) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn1(in *jlexer.Lexer, out *SetUserVerifiedParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		case "isUserVerified":
			out.IsUserVerified = bool(in.Bool())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn1(out *jwriter.Writer, in SetUserVerifiedParams) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	{
		const prefix string = ",\"isUserVerified\":"
		out.RawString(prefix)
		out.Bool(bool(in.IsUserVerified))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v SetUserVerifiedParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn1(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v SetUserVerifiedParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn1(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *SetUserVerifiedParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn1(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *SetUserVerifiedParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn1(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn2(in *jlexer.Lexer, out *SetResponseOverrideBitsParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		case "isBogusSignature":
			out.IsBogusSignature = bool(in.Bool())
		case "isBadUV":
			out.IsBadUV = bool(in.Bool())
		case "isBadUP":
			out.IsBadUP = bool(in.Bool())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn2(out *jwriter.Writer, in SetResponseOverrideBitsParams) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	if in.IsBogusSignature {
		const prefix string = ",\"isBogusSignature\":"
		out.RawString(prefix)
		out.Bool(bool(in.IsBogusSignature))
	}
	if in.IsBadUV {
		const prefix string = ",\"isBadUV\":"
		out.RawString(prefix)
		out.Bool(bool(in.IsBadUV))
	}
	if in.IsBadUP {
		const prefix string = ",\"isBadUP\":"
		out.RawString(prefix)
		out.Bool(bool(in.IsBadUP))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v SetResponseOverrideBitsParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn2(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v SetResponseOverrideBitsParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn2(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *SetResponseOverrideBitsParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn2(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *SetResponseOverrideBitsParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn2(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn3(in *jlexer.Lexer, out *SetCredentialPropertiesParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		case "credentialId":
			out.CredentialID = string(in.String())
		case "backupEligibility":
			out.BackupEligibility = bool(in.Bool())
		case "backupState":
			out.BackupState = bool(in.Bool())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn3(out *jwriter.Writer, in SetCredentialPropertiesParams) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	{
		const prefix string = ",\"credentialId\":"
		out.RawString(prefix)
		out.String(string(in.CredentialID))
	}
	if in.BackupEligibility {
		const prefix string = ",\"backupEligibility\":"
		out.RawString(prefix)
		out.Bool(bool(in.BackupEligibility))
	}
	if in.BackupState {
		const prefix string = ",\"backupState\":"
		out.RawString(prefix)
		out.Bool(bool(in.BackupState))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v SetCredentialPropertiesParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn3(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v SetCredentialPropertiesParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn3(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *SetCredentialPropertiesParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn3(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *SetCredentialPropertiesParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn3(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn4(in *jlexer.Lexer, out *SetAutomaticPresenceSimulationParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		case "enabled":
			out.Enabled = bool(in.Bool())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn4(out *jwriter.Writer, in SetAutomaticPresenceSimulationParams) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	{
		const prefix string = ",\"enabled\":"
		out.RawString(prefix)
		out.Bool(bool(in.Enabled))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v SetAutomaticPresenceSimulationParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn4(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v SetAutomaticPresenceSimulationParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn4(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *SetAutomaticPresenceSimulationParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn4(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *SetAutomaticPresenceSimulationParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn4(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn5(in *jlexer.Lexer, out *RemoveVirtualAuthenticatorParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn5(out *jwriter.Writer, in RemoveVirtualAuthenticatorParams) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v RemoveVirtualAuthenticatorParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn5(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v RemoveVirtualAuthenticatorParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn5(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *RemoveVirtualAuthenticatorParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn5(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *RemoveVirtualAuthenticatorParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn5(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn6(in *jlexer.Lexer, out *RemoveCredentialParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		case "credentialId":
			out.CredentialID = string(in.String())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn6(out *jwriter.Writer, in RemoveCredentialParams) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	{
		const prefix string = ",\"credentialId\":"
		out.RawString(prefix)
		out.String(string(in.CredentialID))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v RemoveCredentialParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn6(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v RemoveCredentialParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn6(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *RemoveCredentialParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn6(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *RemoveCredentialParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn6(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn7(in *jlexer.Lexer, out *GetCredentialsReturns) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "credentials":
			if in.IsNull() {
				in.Skip()
				out.Credentials = nil
			} else {
				in.Delim('[')
				if out.Credentials == nil {
					if !in.IsDelim(']') {
						out.Credentials = make([]*Credential, 0, 8)
					} else {
						out.Credentials = []*Credential{}
					}
				} else {
					out.Credentials = (out.Credentials)[:0]
				}
				for !in.IsDelim(']') {
					var v1 *Credential
					if in.IsNull() {
						in.Skip()
						v1 = nil
					} else {
						if v1 == nil {
							v1 = new(Credential)
						}
						(*v1).UnmarshalEasyJSON(in)
					}
					out.Credentials = append(out.Credentials, v1)
					in.WantComma()
				}
				in.Delim(']')
			}
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn7(out *jwriter.Writer, in GetCredentialsReturns) {
	out.RawByte('{')
	first := true
	_ = first
	if len(in.Credentials) != 0 {
		const prefix string = ",\"credentials\":"
		first = false
		out.RawString(prefix[1:])
		{
			out.RawByte('[')
			for v2, v3 := range in.Credentials {
				if v2 > 0 {
					out.RawByte(',')
				}
				if v3 == nil {
					out.RawString("null")
				} else {
					(*v3).MarshalEasyJSON(out)
				}
			}
			out.RawByte(']')
		}
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v GetCredentialsReturns) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn7(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v GetCredentialsReturns) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn7(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *GetCredentialsReturns) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn7(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *GetCredentialsReturns) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn7(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn8(in *jlexer.Lexer, out *GetCredentialsParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn8(out *jwriter.Writer, in GetCredentialsParams) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v GetCredentialsParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn8(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v GetCredentialsParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn8(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *GetCredentialsParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn8(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *GetCredentialsParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn8(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn9(in *jlexer.Lexer, out *GetCredentialReturns) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "credential":
			if in.IsNull() {
				in.Skip()
				out.Credential = nil
			} else {
				if out.Credential == nil {
					out.Credential = new(Credential)
				}
				(*out.Credential).UnmarshalEasyJSON(in)
			}
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn9(out *jwriter.Writer, in GetCredentialReturns) {
	out.RawByte('{')
	first := true
	_ = first
	if in.Credential != nil {
		const prefix string = ",\"credential\":"
		first = false
		out.RawString(prefix[1:])
		(*in.Credential).MarshalEasyJSON(out)
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v GetCredentialReturns) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn9(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v GetCredentialReturns) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn9(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *GetCredentialReturns) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn9(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *GetCredentialReturns) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn9(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn10(in *jlexer.Lexer, out *GetCredentialParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		case "credentialId":
			out.CredentialID = string(in.String())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn10(out *jwriter.Writer, in GetCredentialParams) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	{
		const prefix string = ",\"credentialId\":"
		out.RawString(prefix)
		out.String(string(in.CredentialID))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v GetCredentialParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn10(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v GetCredentialParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn10(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *GetCredentialParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn10(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *GetCredentialParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn10(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn11(in *jlexer.Lexer, out *EventCredentialUpdated) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		case "credential":
			if in.IsNull() {
				in.Skip()
				out.Credential = nil
			} else {
				if out.Credential == nil {
					out.Credential = new(Credential)
				}
				(*out.Credential).UnmarshalEasyJSON(in)
			}
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn11(out *jwriter.Writer, in EventCredentialUpdated) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	{
		const prefix string = ",\"credential\":"
		out.RawString(prefix)
		if in.Credential == nil {
			out.RawString("null")
		} else {
			(*in.Credential).MarshalEasyJSON(out)
		}
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v EventCredentialUpdated) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn11(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v EventCredentialUpdated) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn11(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *EventCredentialUpdated) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn11(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *EventCredentialUpdated) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn11(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn12(in *jlexer.Lexer, out *EventCredentialDeleted) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		case "credentialId":
			out.CredentialID = string(in.String())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn12(out *jwriter.Writer, in EventCredentialDeleted) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	{
		const prefix string = ",\"credentialId\":"
		out.RawString(prefix)
		out.String(string(in.CredentialID))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v EventCredentialDeleted) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn12(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v EventCredentialDeleted) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn12(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *EventCredentialDeleted) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn12(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *EventCredentialDeleted) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn12(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn13(in *jlexer.Lexer, out *EventCredentialAsserted) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		case "credential":
			if in.IsNull() {
				in.Skip()
				out.Credential = nil
			} else {
				if out.Credential == nil {
					out.Credential = new(Credential)
				}
				(*out.Credential).UnmarshalEasyJSON(in)
			}
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn13(out *jwriter.Writer, in EventCredentialAsserted) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	{
		const prefix string = ",\"credential\":"
		out.RawString(prefix)
		if in.Credential == nil {
			out.RawString("null")
		} else {
			(*in.Credential).MarshalEasyJSON(out)
		}
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v EventCredentialAsserted) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn13(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v EventCredentialAsserted) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn13(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *EventCredentialAsserted) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn13(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *EventCredentialAsserted) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn13(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn14(in *jlexer.Lexer, out *EventCredentialAdded) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		case "credential":
			if in.IsNull() {
				in.Skip()
				out.Credential = nil
			} else {
				if out.Credential == nil {
					out.Credential = new(Credential)
				}
				(*out.Credential).UnmarshalEasyJSON(in)
			}
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn14(out *jwriter.Writer, in EventCredentialAdded) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	{
		const prefix string = ",\"credential\":"
		out.RawString(prefix)
		if in.Credential == nil {
			out.RawString("null")
		} else {
			(*in.Credential).MarshalEasyJSON(out)
		}
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v EventCredentialAdded) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn14(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v EventCredentialAdded) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn14(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *EventCredentialAdded) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn14(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *EventCredentialAdded) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn14(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn15(in *jlexer.Lexer, out *EnableParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "enableUI":
			out.EnableUI = bool(in.Bool())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn15(out *jwriter.Writer, in EnableParams) {
	out.RawByte('{')
	first := true
	_ = first
	if in.EnableUI {
		const prefix string = ",\"enableUI\":"
		first = false
		out.RawString(prefix[1:])
		out.Bool(bool(in.EnableUI))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v EnableParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn15(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v EnableParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn15(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *EnableParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn15(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *EnableParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn15(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn16(in *jlexer.Lexer, out *DisableParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn16(out *jwriter.Writer, in DisableParams) {
	out.RawByte('{')
	first := true
	_ = first
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v DisableParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn16(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v DisableParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn16(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *DisableParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn16(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *DisableParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn16(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn17(in *jlexer.Lexer, out *Credential) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "credentialId":
			out.CredentialID = string(in.String())
		case "isResidentCredential":
			out.IsResidentCredential = bool(in.Bool())
		case "rpId":
			out.RpID = string(in.String())
		case "privateKey":
			out.PrivateKey = string(in.String())
		case "userHandle":
			out.UserHandle = string(in.String())
		case "signCount":
			out.SignCount = int64(in.Int64())
		case "largeBlob":
			out.LargeBlob = string(in.String())
		case "backupEligibility":
			out.BackupEligibility = bool(in.Bool())
		case "backupState":
			out.BackupState = bool(in.Bool())
		case "userName":
			out.UserName = string(in.String())
		case "userDisplayName":
			out.UserDisplayName = string(in.String())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn17(out *jwriter.Writer, in Credential) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"credentialId\":"
		out.RawString(prefix[1:])
		out.String(string(in.CredentialID))
	}
	{
		const prefix string = ",\"isResidentCredential\":"
		out.RawString(prefix)
		out.Bool(bool(in.IsResidentCredential))
	}
	if in.RpID != "" {
		const prefix string = ",\"rpId\":"
		out.RawString(prefix)
		out.String(string(in.RpID))
	}
	{
		const prefix string = ",\"privateKey\":"
		out.RawString(prefix)
		out.String(string(in.PrivateKey))
	}
	if in.UserHandle != "" {
		const prefix string = ",\"userHandle\":"
		out.RawString(prefix)
		out.String(string(in.UserHandle))
	}
	{
		const prefix string = ",\"signCount\":"
		out.RawString(prefix)
		out.Int64(int64(in.SignCount))
	}
	if in.LargeBlob != "" {
		const prefix string = ",\"largeBlob\":"
		out.RawString(prefix)
		out.String(string(in.LargeBlob))
	}
	if in.BackupEligibility {
		const prefix string = ",\"backupEligibility\":"
		out.RawString(prefix)
		out.Bool(bool(in.BackupEligibility))
	}
	if in.BackupState {
		const prefix string = ",\"backupState\":"
		out.RawString(prefix)
		out.Bool(bool(in.BackupState))
	}
	if in.UserName != "" {
		const prefix string = ",\"userName\":"
		out.RawString(prefix)
		out.String(string(in.UserName))
	}
	if in.UserDisplayName != "" {
		const prefix string = ",\"userDisplayName\":"
		out.RawString(prefix)
		out.String(string(in.UserDisplayName))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v Credential) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn17(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v Credential) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn17(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *Credential) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn17(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *Credential) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn17(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn18(in *jlexer.Lexer, out *ClearCredentialsParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn18(out *jwriter.Writer, in ClearCredentialsParams) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v ClearCredentialsParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn18(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v ClearCredentialsParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn18(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *ClearCredentialsParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn18(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *ClearCredentialsParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn18(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn19(in *jlexer.Lexer, out *AddVirtualAuthenticatorReturns) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn19(out *jwriter.Writer, in AddVirtualAuthenticatorReturns) {
	out.RawByte('{')
	first := true
	_ = first
	if in.AuthenticatorID != "" {
		const prefix string = ",\"authenticatorId\":"
		first = false
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v AddVirtualAuthenticatorReturns) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn19(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v AddVirtualAuthenticatorReturns) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn19(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *AddVirtualAuthenticatorReturns) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn19(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *AddVirtualAuthenticatorReturns) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn19(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn20(in *jlexer.Lexer, out *AddVirtualAuthenticatorParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "options":
			if in.IsNull() {
				in.Skip()
				out.Options = nil
			} else {
				if out.Options == nil {
					out.Options = new(VirtualAuthenticatorOptions)
				}
				(*out.Options).UnmarshalEasyJSON(in)
			}
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn20(out *jwriter.Writer, in AddVirtualAuthenticatorParams) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"options\":"
		out.RawString(prefix[1:])
		if in.Options == nil {
			out.RawString("null")
		} else {
			(*in.Options).MarshalEasyJSON(out)
		}
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v AddVirtualAuthenticatorParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn20(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v AddVirtualAuthenticatorParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn20(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *AddVirtualAuthenticatorParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn20(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *AddVirtualAuthenticatorParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn20(l, v)
}
func easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn21(in *jlexer.Lexer, out *AddCredentialParams) {
	isTopLevel := in.IsStart()
	if in.IsNull() {
		if isTopLevel {
			in.Consumed()
		}
		in.Skip()
		return
	}
	in.Delim('{')
	for !in.IsDelim('}') {
		key := in.UnsafeFieldName(false)
		in.WantColon()
		if in.IsNull() {
			in.Skip()
			in.WantComma()
			continue
		}
		switch key {
		case "authenticatorId":
			out.AuthenticatorID = AuthenticatorID(in.String())
		case "credential":
			if in.IsNull() {
				in.Skip()
				out.Credential = nil
			} else {
				if out.Credential == nil {
					out.Credential = new(Credential)
				}
				(*out.Credential).UnmarshalEasyJSON(in)
			}
		default:
			in.SkipRecursive()
		}
		in.WantComma()
	}
	in.Delim('}')
	if isTopLevel {
		in.Consumed()
	}
}
func easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn21(out *jwriter.Writer, in AddCredentialParams) {
	out.RawByte('{')
	first := true
	_ = first
	{
		const prefix string = ",\"authenticatorId\":"
		out.RawString(prefix[1:])
		out.String(string(in.AuthenticatorID))
	}
	{
		const prefix string = ",\"credential\":"
		out.RawString(prefix)
		if in.Credential == nil {
			out.RawString("null")
		} else {
			(*in.Credential).MarshalEasyJSON(out)
		}
	}
	out.RawByte('}')
}

// MarshalJSON supports json.Marshaler interface
func (v AddCredentialParams) MarshalJSON() ([]byte, error) {
	w := jwriter.Writer{}
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn21(&w, v)
	return w.Buffer.BuildBytes(), w.Error
}

// MarshalEasyJSON supports easyjson.Marshaler interface
func (v AddCredentialParams) MarshalEasyJSON(w *jwriter.Writer) {
	easyjsonC5a4559bEncodeGithubComChromedpCdprotoWebauthn21(w, v)
}

// UnmarshalJSON supports json.Unmarshaler interface
func (v *AddCredentialParams) UnmarshalJSON(data []byte) error {
	r := jlexer.Lexer{Data: data}
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn21(&r, v)
	return r.Error()
}

// UnmarshalEasyJSON supports easyjson.Unmarshaler interface
func (v *AddCredentialParams) UnmarshalEasyJSON(l *jlexer.Lexer) {
	easyjsonC5a4559bDecodeGithubComChromedpCdprotoWebauthn21(l, v)
}
