package autofill

// Code generated by cdproto-gen. DO NOT EDIT.

// EventAddressFormFilled emitted when an address form is filled.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/Autofill#event-addressFormFilled
type EventAddressFormFilled struct {
	FilledFields []*FilledField `json:"filledFields"` // Information about the fields that were filled
	AddressUI    *AddressUI     `json:"addressUi"`    // An UI representation of the address used to fill the form. Consists of a 2D array where each child represents an address/profile line.
}
