package performancetimeline

// Code generated by cdproto-gen. DO NOT EDIT.

// EventTimelineEventAdded sent when a performance timeline event is added.
// See reportPerformanceTimeline method.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/PerformanceTimeline#event-timelineEventAdded
type EventTimelineEventAdded struct {
	Event *TimelineEvent `json:"event"`
}
