package deviceaccess

// Code generated by cdproto-gen. DO NOT EDIT.

// EventDeviceRequestPrompted a device request opened a user prompt to select
// a device. Respond with the selectPrompt or cancelPrompt command.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/DeviceAccess#event-deviceRequestPrompted
type EventDeviceRequestPrompted struct {
	ID      RequestID       `json:"id"`
	Devices []*PromptDevice `json:"devices"`
}
