package security

// Code generated by cdproto-gen. DO NOT EDIT.

// EventVisibleSecurityStateChanged the security state of the page changed.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/Security#event-visibleSecurityStateChanged
type EventVisibleSecurityStateChanged struct {
	VisibleSecurityState *VisibleSecurityState `json:"visibleSecurityState"` // Security state information about the page.
}
