package domstorage

// Code generated by cdproto-gen. DO NOT EDIT.

// SerializedStorageKey [no description].
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/DOMStorage#type-SerializedStorageKey
type SerializedStorageKey string

// String returns the SerializedStorageKey as string value.
func (t SerializedStorageKey) String() string {
	return string(t)
}

// StorageID DOM Storage identifier.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/DOMStorage#type-StorageId
type StorageID struct {
	SecurityOrigin string               `json:"securityOrigin,omitempty"` // Security origin for the storage.
	StorageKey     SerializedStorageKey `json:"storageKey,omitempty"`     // Represents a key by which DOM Storage keys its CachedStorageAreas
	IsLocalStorage bool                 `json:"isLocalStorage"`           // Whether the storage is local storage (not session storage).
}

// Item DOM Storage item.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/DOMStorage#type-Item
type Item []string
