package css

// Code generated by cdproto-gen. DO NOT EDIT.

// EventFontsUpdated fires whenever a web font is updated. A non-empty font
// parameter indicates a successfully loaded web font.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/CSS#event-fontsUpdated
type EventFontsUpdated struct {
	Font *FontFace `json:"font,omitempty"` // The web font that has loaded.
}

// EventMediaQueryResultChanged fires whenever a MediaQuery result changes
// (for example, after a browser window has been resized.) The current
// implementation considers only viewport-dependent media features.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/CSS#event-mediaQueryResultChanged
type EventMediaQueryResultChanged struct{}

// EventStyleSheetAdded fired whenever an active document stylesheet is
// added.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/CSS#event-styleSheetAdded
type EventStyleSheetAdded struct {
	Header *StyleSheetHeader `json:"header"` // Added stylesheet metainfo.
}

// EventStyleSheetChanged fired whenever a stylesheet is changed as a result
// of the client operation.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/CSS#event-styleSheetChanged
type EventStyleSheetChanged struct {
	StyleSheetID StyleSheetID `json:"styleSheetId"`
}

// EventStyleSheetRemoved fired whenever an active document stylesheet is
// removed.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/CSS#event-styleSheetRemoved
type EventStyleSheetRemoved struct {
	StyleSheetID StyleSheetID `json:"styleSheetId"` // Identifier of the removed stylesheet.
}
