package backgroundservice

// Code generated by cdproto-gen. DO NOT EDIT.

// EventRecordingStateChanged called when the recording state for the service
// has been updated.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/BackgroundService#event-recordingStateChanged
type EventRecordingStateChanged struct {
	IsRecording bool        `json:"isRecording"`
	Service     ServiceName `json:"service"`
}

// EventBackgroundServiceEventReceived called with all existing
// backgroundServiceEvents when enabled, and all new events afterwards if
// enabled and recording.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/BackgroundService#event-backgroundServiceEventReceived
type EventBackgroundServiceEventReceived struct {
	BackgroundServiceEvent *Event `json:"backgroundServiceEvent"`
}
