package deviceaccess

// Code generated by cdproto-gen. DO NOT EDIT.

// RequestID device request id.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/DeviceAccess#type-RequestId
type RequestID string

// String returns the RequestID as string value.
func (t RequestID) String() string {
	return string(t)
}

// DeviceID a device id.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/DeviceAccess#type-DeviceId
type DeviceID string

// String returns the DeviceID as string value.
func (t DeviceID) String() string {
	return string(t)
}

// PromptDevice device information displayed in a user prompt to select a
// device.
//
// See: https://chromedevtools.github.io/devtools-protocol/tot/DeviceAccess#type-PromptDevice
type PromptDevice struct {
	ID   DeviceID `json:"id"`
	Name string   `json:"name"` // Display name as it appears in a device request user prompt.
}
